package spatial.tree.clustering;

import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.IOException;
import java.util.Vector;

public class KMeansClustering 
{    
    public int k;
    public int n[];
    public int recordsCount;
    public int dimension;
    public double[][] coord;
    public double density[];
    public double centroids[][]; //initial value of centroids
    public double newCentroids[][];
    
    public KMeansClustering(int recordsCount, int dimension, int k) //k = the number of centroids 
    {
        this.k = k;
        centroids = new double[k][dimension]; //dimension attributes for each centroid
        n = new int[k];             
        this.recordsCount = recordsCount;
        this.dimension  = dimension;
    }

    public double getDistance(double[] datum, double[] centroid) 
    {
        double d = 0.0;

        for (int i = 0; i < datum.length - 1; i++) { //calculate distance for each row of data
            d += Math.pow(datum[i] - centroid[i], 2); //Euclidean distance
        }

        return(Math.sqrt(d)); //return distance (note: only returns distances for one row of centroid)
    }

    public int getClosestCentroid(double[] datum)
    {
        double min = Double.MAX_VALUE; //starts with minimum = centroids[0]
        int closestIndex = -1; //-1 because 0 could be a result value

        for (int i = 0; i < centroids.length; i++) { //for each centroid
            double d = getDistance(datum, centroids[i]); // between cluster centroid and object

            if (d < min) { //current distance is less than the minimum distance
                closestIndex = i; //k is now the location of the closest centroid
                min = d; 
            } 
        } 
        return(closestIndex); //returns index of the closest centroid for current datum
    }

    public void printDatum(double[] datum) 
    {
        Vector<Double> v = new Vector<Double>();

        for (int j = 0; j < datum.length; j++) {
            v.add(new Double(datum[j]));
        }

        System.out.println(v);
    }

    public void printCentroids() 
    {
        for (int i = 0; i < centroids.length; i++) {
            printDatum(centroids[i]);
        }

        System.out.println("-------------------");
    }

    public void run() 
    {
        boolean check = false;

        while (check == false) {

            boolean result = true;
            double newCentroids[][] = new double[k][this.dimension];// length = k

            for (int i = 0; i < k; i++) {
                n[i] = 0; //reset value of n[]
            }

            for (int i = 0; i < coord.length; i++) { //ALL data objects
                int count = getClosestCentroid(coord[i]); //gets closest centroid for ALL distances

                for (int j = 0; j < dimension; j++) {
                    newCentroids[count][j] += coord[i][j]; //sums all datum belonging to certain centroid 
                }
                n[count]++; //counts the no. of members of datum that belong to centroid group 
            }

            //finds the average between all datum belonging to certain centroid
            for (int i = 0; i < k; i++) {
                for(int j = 0; j < dimension; j++) {
                    //if(n[i] == 0) n[i] = 1;
                    newCentroids[i][j] = newCentroids[i][j] / n[i];
                }
            }

            //checks if newCentroid values are same as Centroid
            //If they are then there are no more move groups and no more iterations are needed
            for (int i = 0; i < k; i++) {
                for (int j = 0; j < dimension; j++) {
                    if(result == true) {
                        if(newCentroids[i][j] == centroids[i][j]) { //checks for stability
                            check = true;
                            result = true;
                        } else {
                            check = false;
                            result = false; 
                        }
                    }
                }
            }

            centroids = newCentroids;
        }
    }

    public static void main(String args[]) throws IOException 
    {
        KMeansClustering cluster = new KMeansClustering(10, 2, 2);
        
        cluster.coord = new double[10][2];
        cluster.coord[0][0] = 1;
        cluster.coord[0][1] = 1;
        
        cluster.coord[1][0] = 3;
        cluster.coord[1][1] = 1.2;

        cluster.coord[2][0] = 5.1;
        cluster.coord[2][1] = 5;
        
        cluster.coord[3][0] = 5.1;
        cluster.coord[3][1] = 5;

        cluster.coord[4][0] = 5;
        cluster.coord[4][1] = 5.1;
        
        cluster.coord[5][0] = 5.2;
        cluster.coord[5][1] = 5.1;

        cluster.coord[6][0] = 4;
        cluster.coord[6][1] = 4;
        
        cluster.coord[7][0] = 1.1;
        cluster.coord[7][1] = 1.4;

        cluster.coord[8][0] = 4.1;
        cluster.coord[8][1] = 4;
        
        cluster.coord[9][0] = 4.1;
        cluster.coord[9][1] = 4.3;
        
        for (int i = 0; i < cluster.k; i++) { //populate centroids with first k no. of data
            for(int j = 0; j < cluster.dimension; j++) {
                cluster.centroids[i][j] = cluster.coord[i][j];
            }
        }

        cluster.run();
        
        cluster.printCentroids();
    }
}